/*
 * Copyright (c) 2021 LAAS/CNRS
 * All rights reserved.
 *
 * Redistribution  and  use  in  source  and binary  forms,  with  or  without
 * modification, are permitted provided that the following conditions are met:
 *
 *   1. Redistributions of  source  code must retain the  above copyright
 *      notice and this list of conditions.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice and  this list of  conditions in the  documentation and/or
 *      other materials provided with the distribution.
 *
 * THE SOFTWARE  IS PROVIDED "AS IS"  AND THE AUTHOR  DISCLAIMS ALL WARRANTIES
 * WITH  REGARD   TO  THIS  SOFTWARE  INCLUDING  ALL   IMPLIED  WARRANTIES  OF
 * MERCHANTABILITY AND  FITNESS.  IN NO EVENT  SHALL THE AUTHOR  BE LIABLE FOR
 * ANY  SPECIAL, DIRECT,  INDIRECT, OR  CONSEQUENTIAL DAMAGES  OR  ANY DAMAGES
 * WHATSOEVER  RESULTING FROM  LOSS OF  USE, DATA  OR PROFITS,  WHETHER  IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR  OTHER TORTIOUS ACTION, ARISING OUT OF OR
 * IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 *                                           Anthony Mallet on Wed Sep  8 2021
 */

#include "acxsensmvn.h"

#include <err.h>
#include <errno.h>
#include <stdlib.h>
#include <unistd.h>

#include "xsensmvn_c_types.h"

#include "codels.h"


/* --- Task cmd --------------------------------------------------------- */


/** Codel mvn_disconnect of task cmd.
 *
 * Triggered by xsensmvn_stop.
 * Yields to xsensmvn_ether.
 */
genom_event
mvn_disconnect(xsensmvn_remote_s **remote,
               const xsensmvn_localhost_s *localhost,
               const genom_context self)
{
  (void)self;

  if (*remote) {
    mvn_remote_stream(
      (*remote)->fd, false,
      localhost->host, localhost->port, "", mvn_deadline(500));
    close((*remote)->fd);
    free(*remote);
    *remote = NULL;
    warnx("stopped streaming to %s:%s", localhost->host, localhost->port);
  }

  return xsensmvn_ether;
}


/* --- Activity connect ------------------------------------------------- */

/** Codel mvn_connect_start of activity connect.
 *
 * Triggered by xsensmvn_start.
 * Yields to xsensmvn_stream, xsensmvn_getlocalhost.
 * Throws xsensmvn_e_sys.
 */
genom_event
mvn_connect_start(const char host[128], const char port[128],
                  xsensmvn_remote_s **remote,
                  const xsensmvn_localhost_s *localhost,
                  const genom_context self)
{
  xsensmvn_e_sys_detail d;
  xsensmvn_remote_s *r;

  if (*remote)
    mvn_disconnect(remote, localhost, self);

  r = malloc(sizeof(*r));
  if (!r) return mvn_e_sys_error(errno, "malloc", self);

  r->fd =
    mvn_remote_socket(host, port, mvn_deadline(500), d.what, sizeof(d.what));
  if (r->fd < 0) goto err;

  *remote = r;

  return xsensmvn_getlocalhost;

err:
  free(r);
  d.code = EIO;
  return xsensmvn_e_sys(&d, self);
}

/** Codel mvn_connect_getlocalhost of activity connect.
 *
 * Triggered by xsensmvn_getlocalhost.
 * Yields to xsensmvn_stream.
 * Throws xsensmvn_e_sys.
 */
genom_event
mvn_connect_getlocalhost(const xsensmvn_remote_s *remote,
                         xsensmvn_localhost_s *localhost,
                         const genom_context self)
{
  xsensmvn_e_sys_detail d;

  if (localhost->host[0] != '\0') return xsensmvn_stream;

  if (mvn_remote_localhost(
        remote->fd,
        localhost->host, sizeof(localhost->host),
        d.what, sizeof(d.what))) {
    d.code = EIO;
    return xsensmvn_e_sys(&d, self);
  }
  warnx("autoconfigured streaming address %s:%s",
        localhost->host, localhost->port);

  return xsensmvn_stream;
}

/** Codel mvn_connect_stream of activity connect.
 *
 * Triggered by xsensmvn_stream.
 * Yields to xsensmvn_ether.
 * Throws xsensmvn_e_sys.
 */
genom_event
mvn_connect_stream(xsensmvn_remote_s **remote,
                   const xsensmvn_localhost_s *localhost,
                   const genom_context self)
{
  static const char *protos[] = {
    "DgramLinearSegmentKinematics",
    "DgramAngularSegmentKinematics"
  };
  size_t i;

  if (mvn_remote_stream(
        (*remote)->fd, false,
        localhost->host, localhost->port, "", mvn_deadline(500)))
    return mvn_e_sys_error(errno, "unstream", self);

  for(i = 0; i < sizeof(protos)/sizeof(*protos); i++)
    if (mvn_remote_stream(
          (*remote)->fd, true,
          localhost->host, localhost->port, protos[i],
          mvn_deadline(500)))
      return mvn_e_sys_error(errno, "stream", self);

  warnx("streaming to %s:%s", localhost->host, localhost->port);
  return xsensmvn_ether;
}


/* --- Activity disconnect ---------------------------------------------- */

/** Codel mvn_disconnect of activity disconnect.
 *
 * Triggered by xsensmvn_start.
 * Yields to xsensmvn_ether.
 */
/* already defined in task cmd */

